// testlistboxex.c
//
// Demonstrates the ListBoxEx control.
//
// John Findlay
//
// Link with wlib.lib
//


#include <windows.h>
#include <stdio.h>
#include "listboxex.h"
#include "resource.h"

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

#define NUMROWS 22
#define NUMCOLS 5	// be careful to make sure these arrays have enough members
//					   to match NUMCOLS, otherwise, !*&^%$
//char *strRow[NUMCOLS]    = {"Row 111", "Row 222"};
//char *strCol[NUMCOLS]    = {"Col 0", "Col 1"};
//int  colwid[NUMCOLS]     = {80, 110};
//char *strInsert[NUMCOLS] = {"Monday", "Tuesday"};

char *strRow[NUMCOLS]    = {"Row 111", "Row 222", "Row 333", "Row 444", "Row 555"};
char *strCol[NUMCOLS]    = {"Col 0", "Col 1", "Col 2", "Col 3", "Col 4"};
int  colwid[NUMCOLS]     = {80, 110, 110, 110, 110 };
char *strInsert[NUMCOLS] = {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday"};

#define LBX_ID  1200

// The handle for the control.
HWND g_hWndLbx;
HWND hWnd;

char cmpstr1[200];
char cmpstr2[200];

char strMess[200];

int CALLBACK compare( int a, int b, int sign )
{
	// get two strings to compare in cmpstr1 & cmpstr2
	//                        row col
	Lbx_GetItemText( g_hWndLbx, a, 0, cmpstr1, 200 );
	Lbx_GetItemText( g_hWndLbx, b, 0, cmpstr2, 200 );
	if (!sign)
		return strcmp( cmpstr1, cmpstr2 );
	else
		return -strcmp( cmpstr1, cmpstr2 );
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmdLine, int nCmdShow)
{
	MSG					msg;
	WNDCLASS			wc;

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= (WNDPROC)MainWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInstance;
	wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(1));
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= (HBRUSH)(COLOR_BTNFACE + 1);
	wc.lpszMenuName		= MAKEINTRESOURCE(ID_MENU);
	wc.lpszClassName	= "main";

	if ( !RegisterClass(&wc) )
		return 0;

	hWnd = CreateWindowEx( WS_EX_CLIENTEDGE, "main", "ListBoxEx Demo...",
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT,CW_USEDEFAULT,
		620,460,
		NULL,
		(HMENU)0,
		hInstance,
		NULL );

	if ( hWnd == NULL )
	{
		MessageBox( NULL, "Not today!", "Message", MB_OK | MB_APPLMODAL );
		return 0;
	}

	ShowWindow( hWnd, SW_NORMAL );
	UpdateWindow( hWnd );

	while ( GetMessage( &msg, NULL, 0, 0 ) )
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}

LRESULT CALLBACK MainWndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch (message){
	case WM_CREATE:{

		// create the ListBoxEx
		g_hWndLbx = CreateListBoxEx(
			LBXS_EX_FULLROWSELECT | LBXS_EX_GRIDLINES | LBXS_EX_NOTIFYDBCLICK,
			LBXS_BORDER | LBXS_SHOWSELALWAYS | LBXS_SUNKEN,
			20, 20, 300, 300, hWnd, NUMCOLS, LBX_ID );

		// Check for the ListBoxEx window
		if ( g_hWndLbx == (HWND)0 ){
			MessageBox( GetActiveWindow(), "Oops! No ListBoxEx today.", "ListBoxEx Message", MB_OK | MB_APPLMODAL );
			return -1;
		}

		Lbx_SetColHeadTxtAll( g_hWndLbx, NUMCOLS, strCol );	// set all header texts

		Lbx_SetColWidthAll( g_hWndLbx, NUMCOLS, colwid );		// set all column widths

		Lbx_ShowWindow( g_hWndLbx, SW_SHOW );				// make it visible

		return 0;
	}

	case WM_SETFOCUS:
		SetFocus( g_hWndLbx ); // causes the focus to return to the ListBoxEx control
		break;                 // when we receive the focus from the system.

	case LBXN_HDRCLICK:
		sprintf( strMess, "Control ID -> %d Header Button -> %d", wParam, lParam );
		MessageBox( GetActiveWindow(), strMess, "Clicked or Double Clicked Item", MB_OK | MB_APPLMODAL );
		break;

	case LBXN_CLICK:
		sprintf( strMess, "Control ID -> %d Clicked Item -> %d", wParam, lParam );
		MessageBox( GetActiveWindow(), strMess, "Clicked or Double Clicked Item", MB_OK | MB_APPLMODAL );
		break;

	case WM_COMMAND:
		switch(LOWORD(wParam)){
		case IDM_GETITEM:{
			Lbx_GetItemText( g_hWndLbx, 4, 2, strMess, 200 );
			MessageBox(0, strMess, "Row 4 Col 2", MB_OK | MB_APPLMODAL);
			break;
		}

		case IDM_SETSELECTED:
			Lbx_SetSelected( g_hWndLbx, 4, TRUE );
			break;

		case IDM_UNSETSELECTED:
			Lbx_SetSelected( g_hWndLbx, 4, FALSE );
			break;

		case IDM_SCROLL: //        dx  dy
			Lbx_Scroll( g_hWndLbx, 30, 100 );
			break;

		case IDM_SORT:
			Lbx_SortRows( g_hWndLbx );
			break;

		case IDM_SORTEX:
			Lbx_SortRowsEx( g_hWndLbx, compare, 1 );
			break;

		case IDM_EMPTYLIST:
			Lbx_DeleteAllRows( g_hWndLbx );
			break;

		case IDM_INSERT:
			Lbx_InsertRowAll( g_hWndLbx, NUMCOLS, 4, strInsert );
			break;

		case IDM_GETCURSEL:
			sprintf( strMess, "%d", Lbx_GetCurSel( g_hWndLbx ));
			MessageBox( GetActiveWindow(), strMess, "Message", MB_OK | MB_APPLMODAL );
			break;

		case IDM_GETSELCOUNT:
			sprintf( strMess, "%d", Lbx_GetSelCount( g_hWndLbx ));
			MessageBox( GetActiveWindow(), strMess, "Message", MB_OK | MB_APPLMODAL );
			break;

		case IDM_GETROW:{
			char * str[NUMCOLS];

			for (int i = 0; i<NUMCOLS; i++)
				str[i] = malloc( 16 );
			//                                 row
			Lbx_GetRowText( g_hWndLbx, NUMCOLS, 2, str, 16 );

			strcpy(strMess,  str[0]);
			strcat(strMess, "\n" );  // line break

			for (int i = 1; i<NUMCOLS; i++){
				strcat(strMess,  str[i]);
				strcat(strMess, "\n" );  // line break
			}

			for (int i = 0; i<NUMCOLS; i++)
				free(str[i]);

			MessageBox( GetActiveWindow(), strMess, "GetRowText", MB_OK | MB_APPLMODAL );
			}
			break;

		case IDM_GETROWBUFFER:{
			char tmpbuf[20];	// working buffer
			char store[120];	// main buffer
			//                               numcols row                   182 == 
			Lbx_GetRowTextBuffer( g_hWndLbx, NUMCOLS, 3, store, 1000, tmpbuf, 100, 182 );
			MessageBox( GetActiveWindow(), store, "GetRowTextBuffer", MB_OK | MB_APPLMODAL );
			}
			break;

		case IDM_DELETEITEM:
			Lbx_DeleteRow( g_hWndLbx, 4 );
			break;

		case IDM_GETCOLWIDTH:
			sprintf( strMess, "%d", Lbx_GetColWidth( g_hWndLbx, 1 ) );
			MessageBox( GetActiveWindow(), strMess, "Get Column Width", MB_OK | MB_APPLMODAL );
			break;

		case IDM_SETFONT:
			Lbx_SetFont( g_hWndLbx, GetStockObject(SYSTEM_FIXED_FONT) );
			break;

		case IDM_SETBKCOLOR:
			Lbx_SetBkColor( g_hWndLbx, RGB( 192, 192, 192 ) );
			break;

		case IDM_SETTEXTCOLOR:
			Lbx_SetTextColor( g_hWndLbx, RGB(228, 0, 0 ) );
			break;

		case IDM_ITEMCOUNT:
			sprintf( strMess, "%d", Lbx_GetRowCount( g_hWndLbx) );
			MessageBox( GetActiveWindow(), strMess, "GetItemCount", MB_OK | MB_APPLMODAL );
			break;

		case IDM_GETTOPVISITEM:
			sprintf( strMess, "%d", Lbx_TopVisibleRow( g_hWndLbx) );
			MessageBox( GetActiveWindow(), strMess, "Get Top Cur Selection", MB_OK | MB_APPLMODAL );
			break;

		case IDM_GETSTRINGWIDTH:
			sprintf( strMess, "Hello is %d pixels wide", Lbx_GetStringWidth( g_hWndLbx, "Hello") );
			MessageBox( GetActiveWindow(), strMess, "Width of [Hello] in pixels", MB_OK | MB_APPLMODAL );
			break;

		case IDM_FINDITEM:
			sprintf( strMess, "%d", Lbx_FindRow(  g_hWndLbx, -1, "Row 04") );
			MessageBox( GetActiveWindow(), strMess, "Row 4 found at index", MB_OK | MB_APPLMODAL );
			break;

		case IDM_FILLLIST:{
			Lbx_DeleteAllRows( g_hWndLbx );
			int cnt = 0;
			for (int i = 0; i<NUMROWS; i++){
				for (int j = 0; j<NUMCOLS; j++){
					sprintf(strRow[j]+4, "%02d", 21-i);
					cnt++;
				}
				Lbx_AddRowAll( g_hWndLbx, NUMCOLS, strRow );
			}

		}
		break;

		case IDM_EXIT:
			PostMessage( hWnd, WM_CLOSE, 0, 0 );
			break;
		}
		break;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	}

	return DefWindowProc( hWnd, message, wParam, lParam );
}
