// picbox_test.c
//
// Demonstrates the PictureBox Control.
//
// John Findlay
//
// Because lcclnk is a one pass linker, the order one links the libs
// is important - Link in this order - wlib.lib ole32.lib olepro32.lib uuid.lib
//
// See the html doc for details.
//
#include <windows.h>
#include "resource.h"
#include "picturebox.h"

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

HWND 	  hWnd;
HINSTANCE hInst;

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrev, LPSTR lpCmdLine, int nCmdShow)
{
	MSG			msg;
	WNDCLASS	wc;

	hInst = hInstance;

	wc.style			= CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc		= (WNDPROC)MainWndProc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= hInst;
	wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(1));
	wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= (HBRUSH)(COLOR_BTNFACE + 1);
	wc.lpszMenuName		= MAKEINTRESOURCE(ID_MENU);
	wc.lpszClassName	= "main";

	if (!RegisterClass(&wc))
		return 0;

	// create main window
	hWnd = CreateWindowEx(WS_EX_CLIENTEDGE, "main", "PictureBoxEx Demo...",
		WS_OVERLAPPEDWINDOW,
		40,40,
		620,460,
		NULL,
		(HMENU)0,
		hInstance,
		NULL);

	if (hWnd == NULL)
	{
		MessageBox(NULL, "Not today!", "Message", MB_OK | MB_APPLMODAL);
		return 0;
	}
	HWND h;

	InitPictureBox(); // you need to use InitPictureBox() if using CreateWindow first.
					  // CreatePictureBox automatically calls InitPictureBox().

	// Create picbox's and load the images from the resources.
	char cables[] = "#2";
	h = CreateWindow(PICTUREBOX, cables,
			WS_CHILD | PXS_GIF | WS_VISIBLE | PXS_BORDER | PXS_FITBOX,
			20, 10, 80, 80, hWnd, (HMENU)1, hInstance, 0);

//	SendMessage(h, PXM_CURSOR, 0, (LPARAM)LoadCursor(NULL, IDC_WAIT));
	PicBox_Cursor(h, LoadCursor(NULL, IDC_NO));

	char globe[] = "#1";
	h = CreatePictureBox(WS_VISIBLE | PXS_JPG | PXS_SUNKEN | PXS_FITIMAGE,
			globe, 140, 10, 80, 80, hWnd, 2);

	char land[] = "#1";
	h = CreatePictureBox(WS_VISIBLE | PXS_BITMAP | PXS_BORDER | PXS_FITIMAGE,
			land, 20, 160, 80, 80, hWnd, 3);

	char bigico[] = "#1";
	h = CreatePictureBox(WS_VISIBLE | PXS_ICON | PXS_FITIMAGE,
		bigico, 240, 240, 80, 80, hWnd, 4);

	char moon[] = "#3";
	h = CreatePictureBox(WS_VISIBLE | PXS_JPG | PXS_SUNKEN | PXS_FITIMAGE,
		moon, 300, 10, 80, 80, hWnd, 5);

	ShowWindow(hWnd, SW_NORMAL);
	UpdateWindow(hWnd);

	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	ExitPictureBox();

	return msg.wParam;
}

BOOL _stdcall AboutDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message){
	case WM_COMMAND:
		if(LOWORD(wParam) == IDOK)
			EndDialog(hDlg, TRUE);
		break;

	case WM_CLOSE:
		EndDialog(hDlg, TRUE);
		return 0;

	case WM_DESTROY:
		EndDialog(hDlg, TRUE);
		return TRUE;

	default:
		return 0;
	}
	return 0;
}

LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message){
	case WM_COMMAND:
		switch(LOWORD(wParam)){
		case IDM_CHANGE:
			SendMessage(GetDlgItem(hWnd, 5), PXM_SETIMAGE, PXS_BITMAP, (LPARAM)"land.bmp");
//			PicBox_Image(GetDlgItem(hWnd, 5), PXS_BITMAP, "land.bmp");
			break;

		case IDM_STYLE:																//    cx  cy
//			SendMessage(GetDlgItem(hWnd, 5), PXM_STYLE, PXS_BORDER | PXS_FITBOX, MAKELONG(80, 60));
			PicBox_Style(GetDlgItem(hWnd, 5), PXS_BORDER | PXS_FITBOX, MAKELONG(80, 60));
			break;

		case IDM_DIAG:
			InitPictureBox(); // use init before open dialog with a PictureBox control.
			DialogBox(hInst, MAKEINTRESOURCE(DLG_100), hWnd, AboutDlg);
			break;

		case IDM_EXIT:
			PostMessage(hWnd, WM_CLOSE, 0, 0);
			break;
		}
		break;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}


